/*
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 *  This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: iaom <zhangpengfei@kylinos.cn>
 */

#ifndef UKUI_NOTIFICATION_POPUP_NOTIFICATION_H
#define UKUI_NOTIFICATION_POPUP_NOTIFICATION_H
#include "ukui-notification_global.h"
#include <QObject>
#include <QDateTime>
#include <QList>
#include <QString>
#include <QUrl>
#include <QVariantMap>
#include <QImage>
#include <QList>
#include <QPair>
namespace UkuiNotification {
class PopupNotificationPrivate;
typedef QList<QPair<QString, QString>> ActionList;
class UKUINOTIFICATION_EXPORT PopupNotification
{
    friend class NotificationSettingsTest;
    Q_GADGET
    Q_PROPERTY(uint id READ id)
    Q_PROPERTY(QString applicationName READ applicationName WRITE setApplicationName)
    Q_PROPERTY(QString applicationIconName READ applicationIconName WRITE setApplicationIconName)
    Q_PROPERTY(QString summary READ summary WRITE setSummary)
    Q_PROPERTY(QString body READ body WRITE setBody)
    Q_PROPERTY(QDateTime createdTime READ createdTime)
    Q_PROPERTY(bool hasDefaultAction READ hasDefaultAction)
    Q_PROPERTY(ActionList actions READ actions)
    Q_PROPERTY(int timeout READ timeout WRITE setTimeout)
    Q_PROPERTY(bool enableActionIcons READ enableActionIcons)
    Q_PROPERTY(QImage image READ image)
    Q_PROPERTY(QString icon READ icon)
    Q_PROPERTY(bool resident READ resident)
    Q_PROPERTY(bool transient READ transient)
    Q_PROPERTY(QString soundFile READ soundFile)
    Q_PROPERTY(QString soundName READ soundName)
    Q_PROPERTY(bool suppressSound READ suppressSound)
    Q_PROPERTY(QString category READ category)
    Q_PROPERTY(Urgency urgency READ urgency WRITE setUrgency)
    Q_PROPERTY(QString display READ display)
    Q_PROPERTY(QStringList actionState READ actionState)
    Q_PROPERTY(bool noFold READ noFold)
    Q_PROPERTY(int popupTimeout READ popupTimeout)

public:
/**
 * The notification urgency.
 */
    enum Urgency {
        LowUrgency =  0,
        NormalUrgency = 1,
        CriticalUrgency = 2,
    };
    Q_ENUM(Urgency)

    explicit PopupNotification(uint id = 0);
    PopupNotification(const PopupNotification &other);
    PopupNotification &operator=(const PopupNotification &other);
    PopupNotification &operator=(PopupNotification &&other) Q_DECL_NOEXCEPT;
    ~PopupNotification();

    uint id() const;
    QString applicationName() const;
    void setApplicationName(const QString &applicationName);

    QString applicationIconName() const;
    void setApplicationIconName(const QString &applicationIconName);

    QString summary() const;
    void setSummary(const QString &summary);

    QString body() const;
    void setBody(const QString &body);

    bool hasDefaultAction() const;
    QString defaultActionLabel();
    void setActions(const QStringList &actions);
    ActionList actions() const;
    QStringList actionState() const;

    QVariantMap hints() const;
    void setHints(const QVariantMap &hints);

    int timeout() const;
    void setTimeout(int timeout);

    QDateTime createdTime() const;
    //m_hints
    /**
     * 使用图标显示action
     * @return
     */
    bool enableActionIcons() const;
    /**
     * 分类
     * @return
     */
    QString category() const;
    /**
     * desktop名称（不带.desktop后缀）
     * @return
     */
    QString desktopEntry() const;
    /**
     * 对应image-data字段
     * @return
     */
    QImage image() const;
    /**
     * 当加载image-data失败，或通过image-path加载image失败时（image-path不是URL）,icon会被设置为image-path数据
     * @return
     */
    QString icon() const;
    /**
     * 是否持续显示，不会自动移除，直到被用户手动或发送者删除
     * @return
     */
    bool resident() const;
    /**
     * 通知发送时播放的声音文件路径
     * @return
     */
    QString soundFile() const;
    /**
     * 声音主题中的声音名称
     * @return
     */
    QString soundName() const;
    /**
     * 是否静音
     * @return
     */
    bool suppressSound() const;
    /**
     * 暂时通知（会自动移除）
     * @return
     */
    bool transient() const;

    Urgency urgency() const;
    void setUrgency(Urgency urgency);
    /**
     * 通知发送者所在的display
     * @return
     */
    QString display() const;
    /**
     * 是否不折叠
     * @return
     */
    bool noFold() const;
    /**
     * 弹窗持续时间
     * @return
     */
    int popupTimeout() const;

private:
    PopupNotificationPrivate *d = nullptr;
};
}

#endif //UKUI_NOTIFICATION_POPUP_NOTIFICATION_H
